"""
@author: IOB

Create and test random forest models. Toggle "hyperparameter_tuning" variable to retune hyperparameters or leave as 'off' to use 
previous settings.
"""

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from sklearn.ensemble import RandomForestRegressor
from sklearn.metrics import mean_absolute_error as mae
from sklearn.metrics import r2_score
from sklearn.model_selection import (LeaveOneOut, RandomizedSearchCV,
                                     RepeatedKFold, cross_val_predict,
                                     cross_validate, train_test_split)
from sklearn.preprocessing import StandardScaler
plt.style.use('seaborn-white')
plt.rcParams.update({'font.size': 16})
plt.rcParams["font.family"] = "Arial"
plt.rcParams['axes.spines.right'] = False
plt.rcParams['axes.spines.top'] = False

hyperparameter_tuning = 'off' #Change to on or off to perform hyperparameter tuning or not

# Data prep

df = pd.read_csv('./data/CannaLit.csv').dropna()
# df_descriptors = pd.read_csv('./data/straw_models_24.csv')

y, x = df.loc[:, 'yield'],df.iloc[:, 1:-2] #For normal models
# y, x = df.loc[:, 'yield'],df_descriptors #For straw model validation
X_train_raw, X_test_raw, y_train, y_test = train_test_split(x, y,test_size=0.2, random_state=25)

scaler = StandardScaler().fit(X_train_raw)
X_train = scaler.transform(X_train_raw)
X_test = scaler.transform(X_test_raw)

# Hyperparameter tuning
n_estimators = range(1,200)
max_depth = range(1,200)
min_samples_split = range(2,10)
min_samples_leaf = range(1,10)

random_grid = {'n_estimators': n_estimators,
               'max_depth': max_depth,
               'min_samples_split': min_samples_split,
               'min_samples_leaf': min_samples_leaf,
               }

regressor = RandomForestRegressor()
if hyperparameter_tuning == 'on':
    hp_tune = RandomizedSearchCV(estimator = regressor, param_distributions = random_grid, cv=10, n_iter=150,scoring='r2', random_state=25, verbose=3)
    hp_tune.fit(X_train, y_train)
    print(hp_tune.best_params_)
    rf_random = hp_tune.best_estimator_

elif hyperparameter_tuning == 'off':
    rf_random = RandomForestRegressor(n_estimators=62,min_samples_split=4,min_samples_leaf=1,max_depth=99).fit(X_train,y_train)

train_pred = rf_random.predict(X_train)
test_pred = rf_random.predict(X_test)

#Cross-validate
rep_cv = RepeatedKFold(n_splits=10,n_repeats=10)
cv_model = cross_validate(
    rf_random,
    X_train,
    y_train,
    cv = rep_cv,
    scoring=('r2','neg_mean_absolute_error')
)
cv_r2 = (np.average(cv_model['test_r2']))

#LOO
loo = LeaveOneOut()
loo_model = cross_val_predict(
    rf_random,
    X_train,
    y_train,
    cv = loo,
)
loo_r2 = r2_score(y_train,loo_model)  

print('Train MAE ' + str(np.round(mae(y_train,train_pred),2)))
print('Train R^2 ' + str(np.round(r2_score(y_train,train_pred),2)))
print('Test MAE ' + str(np.round(mae(y_test,test_pred),2)))
print('Test R^2 ' + str(np.round(r2_score(y_test,test_pred),2)))
print('10-fold CV ' + str(np.round(cv_r2,2)))
print('LOO ' + str(np.round(loo_r2,2)))

plt.scatter(y_train, train_pred, label='Training Set', c='#8c8c8c')
plt.scatter(y_test, test_pred, label='Test Set', c='#c44f51', marker='x')
plt.xlabel('Measured Yield (%)')
plt.ylabel('Predicted Yield (%)')
plt.legend(loc="lower right")
plt.title('RandomForestRegressor')
plt.xlim([-5,105])
plt.ylim([-5,105])
# plt.savefig('./figure.svg',bbox_inches='tight')
plt.show()
