"""
@author: IOB

Create and test XGBoost models. Toggle "hyperparameter_tuning" variable to retune hyperparameters or leave as 'off' to use 
previous settings.
"""

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from xgboost import XGBRegressor
from sklearn.metrics import mean_absolute_error as mae
from sklearn.metrics import r2_score
from sklearn.model_selection import (LeaveOneOut, RandomizedSearchCV,
                                     RepeatedKFold, cross_val_predict,
                                     cross_validate, train_test_split)
from sklearn.preprocessing import StandardScaler
plt.style.use('ggplot')
plt.rcParams.update({'font.size': 16})
plt.rcParams["font.family"] = "Helvetica"

hyperparameter_tuning = 'off' #Change to on or off to perform hyperparameter tuning or not

# Data prep

df = pd.read_csv('./data/CannaLit.csv').dropna()
y, x = df.loc[:, 'yield'],df.iloc[:, 1:-2]
X_train_raw, X_test_raw, y_train, y_test = train_test_split(x, y,test_size=0.2, random_state=25)

scaler = StandardScaler().fit(X_train_raw)
X_train = scaler.transform(X_train_raw)
X_test = scaler.transform(X_test_raw)

# Hyperparameter tuning
eta = np.divide(range(1,10),10)
gamma = range(0,10)
max_depth = range(0,20)
subsample = np.divide(range(1,10),10)
min_child_weight = range(0,5)

random_grid = {'eta': eta,
               'gamma': gamma,
               'max_depth': max_depth,
               'subsample': subsample,
               'min_child_weight': min_child_weight
               }

regressor = XGBRegressor()
if hyperparameter_tuning == 'on':
    hp_tune = RandomizedSearchCV(estimator = regressor, param_distributions = random_grid, cv=10, n_iter=150,scoring='r2', random_state=25, verbose=3)
    hp_tune.fit(X_train, y_train)
    print(hp_tune.best_params_)
    xgb = hp_tune.best_estimator_

elif hyperparameter_tuning == 'off':
    xgb = XGBRegressor(subsample=0.8,min_child_weight=2,max_depth=5,gamma=7,eta=0.1).fit(X_train,y_train)

train_pred = xgb.predict(X_train)
test_pred = xgb.predict(X_test)

#Cross-validate
rep_cv = RepeatedKFold(n_splits=10,n_repeats=10)
cv_model = cross_validate(
    xgb,
    X_train,
    y_train,
    cv = rep_cv,
    scoring=('r2','neg_mean_absolute_error')
)
cv_r2 = (np.average(cv_model['test_r2']))

#LOO
loo = LeaveOneOut()
loo_model = cross_val_predict(
    xgb,
    X_train,
    y_train,
    cv = loo,
)
loo_r2 = r2_score(y_train,loo_model)  

print('Train MAE ' + str(np.round(mae(y_train,train_pred),2)))
print('Train R^2 ' + str(np.round(r2_score(y_train,train_pred),2)))
print('Test MAE ' + str(np.round(mae(y_test,test_pred),2)))
print('Test R^2 ' + str(np.round(r2_score(y_test,test_pred),2)))
print('10-fold CV ' + str(np.round(cv_r2,2)))
print('LOO ' + str(np.round(loo_r2,2)))

plt.scatter(y_train, train_pred, label='Training Set', c='#8c8c8c')
plt.scatter(y_test, test_pred, label='Test Set', c='#c44f51', marker='x')
plt.xlabel('Measured Yield')
plt.ylabel('Predicted Yield')
plt.legend(loc="lower right")
plt.title('XGBoostRegressor')
plt.xlim([-5,105])
plt.ylim([-5,105])
# plt.savefig('./figure.svg', bbox_inches='tight')
plt.show()
